#!/usr/bin/perl

############################### This program is free software; you can redistribute it and/or modify
############################### it under the terms of the GNU General Public License as published by
############################### the Free Software Foundation; either version 2 of the License, or
############################### (at your option) any later version.
############################### 
############################### This program is distributed in the hope that it will be useful,
############################### but WITHOUT ANY WARRANTY; without even the implied warranty of
############################### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
############################### GNU General Public License for more details.
############################### 
############################### You should have received a copy of the GNU General Public License
############################### along with this program; if not, write to the Free Software
############################### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
###############################
###############################
###############################
############################### Copyright 2006 Damon P. Little

#start
use DBI;
my $db = "barcode";
my $host = "localhost";
my $id = "classif";
my $pwd = "classif";
$loci = "loci";
$sequences = "sequences";
$taxa = "taxa";
$oligos = "motifs";
$frequency = "freq10";

#get options
if($ARGV[0] eq "-e"){
	$option = "=";
	} elsif ($ARGV[0] eq "-l"){
		$option = "LIKE";
		} 
if($ARGV[1]){
	$locus = $ARGV[1]; 
	} else {
		$locus = "NULL";
		}
if($ARGV[2]){
	$number = $ARGV[2]
	} else {
		$number = "-a";
		}
	
#error trap
if($locus eq "NULL"){
	print "A Perl script for updating the motif count in the mysql database.\n";
	print "This script should really be a part of fst2mysql.pl.\n";
	print "USAGE: count-patterns.pl -e[-l] locus -a[-h]\n";
	print "where -e specifies exact locus match; -l LIKE\n";
	print "-a all sequences; -h high scoring sequences\n";
	} else {
		
		#connect
		my $dbh = DBI->connect("DBI:mysql:$db:$host",$id,$pwd);
		if(!defined $dbh){
			die "Can't connect: $DBI::errstr \n";
			}
		
		#olig query
		$mystr = "SELECT motif_id, motif FROM $oligos ORDER BY motif_id";	
		$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute || die "can't execute the query: $sth->errstr";
		my $oligo_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		
		#seq query
		$mystr = "SELECT $sequences.genbank_id, $taxa.genus, $taxa.species, $taxa.subsp, $taxa.var, $sequences.sequence 
				  FROM $sequences, $taxa, $loci 
				  WHERE $taxa.taxon_id=$sequences.taxon_id AND $sequences.locus_id=$loci.locus_id 			  
				  AND $loci.name_short $option ?			  
				  ORDER BY $taxa.genus, $taxa.species, $sequences.score DESC";	
		$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute ($locus) || die "can't execute the query: $sth->errstr";
		my $table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		
		#locus query
		$mystr = "SELECT locus_id FROM $loci WHERE $loci.name_short $option ?";	
		$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute ($locus) || die "can't execute the query: $sth->errstr";
		my $locus_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		$locusID = $locus_table->[0][0];
		
		$last_genus = "";
		$last_species = "";
		$last_seq = "";
							
		for $k (0 .. $#{$table}){					
			$seq_id = $table->[$k][0];
			$genus = $table->[$k][1];
			$species = $table->[$k][2];
			$subsp = $table->[$k][3];
			$var = $table->[$k][4];
			$seq = uc($table->[$k][5]);
		
			if((($genus eq $last_genus) && ($species eq $last_species) && ($number eq "-a") && ($seq ne $last_seq)) || ($genus ne $last_genus) || ($species ne $last_species)){
				$revcom = reverse($seq);
				$revcom =~ tr/ACGTNVDBHWMRKSY/TGCANBHVDWKYMSR/;
				$seq .= " " . $revcom;
				for $j (0 .. $#{$oligo_table}){
					$oligoID = $oligo_table->[$j][0];
					$oligo = $oligo_table->[$j][1];
					if($seq =~ /$oligo/){
						$mystr = "INSERT INTO $frequency (locus_id, motif_id, hits) 
								  VALUES (?, ?, \"1\") 
								  ON DUPLICATE KEY UPDATE hits=hits+1";	
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute($locusID, $oligoID) || die "can't execute the query: $sth->errstr";
						}
					}
				}
			$last_genus = $genus;
			$last_species = $species;
			$last_seq = $seq;
			}
		$rc = $sth->finish;
		$dbh->disconnect;
		}	


exit;
