#!/usr/bin/perl

############################### This program is free software; you can redistribute it and/or modify
############################### it under the terms of the GNU General Public License as published by
############################### the Free Software Foundation; either version 2 of the License, or
############################### (at your option) any later version.
############################### 
############################### This program is distributed in the hope that it will be useful,
############################### but WITHOUT ANY WARRANTY; without even the implied warranty of
############################### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
############################### GNU General Public License for more details.
############################### 
############################### You should have received a copy of the GNU General Public License
############################### along with this program; if not, write to the Free Software
############################### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
###############################
###############################
###############################
############################### Copyright 2006 Damon P. Little

print "A Perl script for making FASTA files into the barcode database.\n";
print "Assumes that it is either GenBank FASTA, or FASTA with id_genus_species.\n";
print "For non-GenBank FASTA short locus name is a required argument.\n";
print "USAGE: fst2mysql.pl in_file division [short_locus_name]\n";

use DBI;
my $db = "barcode";
my $host = "localhost";
my $id = "barcode";
my $pwd = "barcode";
$taxa = "taxa";
$loci = "loci";
$sequences = "sequences";

my $dbh = DBI->connect("DBI:mysql:$db:$host",$id,$pwd);
if(!defined $dbh) {
	die "Can't connect: $DBI::errstr \n";
	}
		
if (-e "$ARGV[0]"){ 
	open(INFILE, "$ARGV[0]") || die("Could not open in file!");
	
	$division = $ARGV[1];
	$insert = 0;
	$update = 0;
	$seq = "";
	$label = "";

	foreach $line (<INFILE>){
		chomp($line);
		if(length($line)){
			@temp_line = split("", $line);
			if(($temp_line[0] eq ">") || ($temp_line[0] eq "#" )){ # if label
				#process last seq
				process_data($label, $seq);
				$seq = "";
				#get new label
				$label = $line;
				} else { #if data
					$seq .= $line;
					} 
			}
		}
	#process the very last one...
	process_data($label, $seq);
	close(INFILE);
	$rc = $sth->finish;
	$dbh->disconnect;	
	print "$insert records input; $update records updated\n";
	}

sub process_data($){
	my $label = $_[0];
	my $seq = $_[1];
	if((length($seq)) && (length($label))){
		
		#clean sequence
		$seq = uc($seq);
		$seq =~ tr/ACGTNVDBHWMRKSY//cd;
		
		#score
		$score = 4 * ($seq =~ tr/ATGC/ATGC/);
		$score += 2 * ($seq =~ tr/WMRKSY/WMRKSY/);
		$score += 1 * ($seq =~ tr/VDBH/VDBH/);
	
		#GenBank?
		if($label =~ m/\|/){
			#$genbank = "yes";
			#parse label
			@junk = split(/\|/, $label);
			
			#get accession
			@temp = split(/\./, $junk[3]);
			$genbank_id = $temp[0];
			$genbank_version = $temp[1];
			
			@junk = split(" ", $label);
			$genus = $junk[1];
			$species = $junk[2];
			if(($junk[3] eq "var.") || ($junk[3] eq "subsp.")){
				$start = 5;
				$subsp = "";
				$var = "";
				if($junk[3] eq "var."){
					$var = $junk[4];
					} else {
						$subsp = $junk[4];
						}	
				} else {
					$start = 3;	
					}
			
			#get long locus name
			$name_long = "";
			$k = 0;
			foreach $value(@junk){
				if($k >= $start){
					$name_long .= $value . " ";
					}
				$k++;
				}
			
			#input label	
			#test if data exists
			$mystr = "SELECT seq_id, genbank_version FROM $sequences WHERE genbank_id = ?";
			$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
			$rv = $sth->execute ($genbank_id) || die "can't execute the query: $sth->errstr";
			my $test_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
			if((($#{$test_table}) > -1)){ #if updated data
				if($test_table->[0][1] < $genbank_version){
					$seq_id = $test_table->[0][0];
					$mystr = "UPDATE $sequences SET genbank_version = ?, sequence = ?, score = ? WHERE seq_id = ?";
					$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
					$rv = $sth->execute ($genbank_version, $seq, $score, $seq_id) || die "can't execute the query: $sth->errstr";
					$update++;
					}
				} else { #not in database
					#look for taxon
					if($junk[3] eq "var."){
						$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ? AND var = ?";
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($genus, $species, $var) || die "can't execute the query: $sth->errstr";
						my $division_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
						if(($#{$division_table}) > -1){ #if exists...	
							$division_id = $division_table->[0][0]; 
							} else {#insert taxon
								$mystr = "INSERT INTO $taxa (taxon_id, genus, species, subsp, var, division) VALUES (NULL, ?, ?, NULL, ?, ?)";
								$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
								$rv = $sth->execute ($genus, $species, $var, $division) || die "can't execute the query: $sth->errstr";
								$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ? AND var = ?";
								$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
								$rv = $sth->execute ($genus, $species, $var) || die "can't execute the query: $sth->errstr";
								my $division_second_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
								$division_id = $division_second_table->[0][0]; 
								}
						}
					if($junk[3] eq "subsp."){
						$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ? AND subsp = ?";
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($genus, $species, $subsp) || die "can't execute the query: $sth->errstr";
						my $division_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
						if(($#{$division_table}) > -1){ #if exists...	
							$division_id = $division_table->[0][0]; 
							} else {#insert taxon
								$mystr = "INSERT INTO $taxa (taxon_id, genus, species, subsp, var, division) VALUES (NULL, ?, ?, ?, NULL, ?)";
								$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
								$rv = $sth->execute ($genus, $species, $subsp, $division) || die "can't execute the query: $sth->errstr";
								$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ? AND subsp = ?";
								$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
								$rv = $sth->execute ($genus, $species, $subsp) || die "can't execute the query: $sth->errstr";
								my $division_second_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
								$division_id = $division_second_table->[0][0]; 
								}
						}
					if(($junk[3] ne "var.") && ($junk[3] ne "subsp.")){
						$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ? AND subsp IS NULL AND var IS NULL";
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($genus, $species) || die "can't execute the query: $sth->errstr";
						my $division_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
						if(($#{$division_table}) > -1){ #if exists...	
							$division_id = $division_table->[0][0]; 
							} else {#insert taxon
								$mystr = "INSERT INTO $taxa (taxon_id, genus, species, subsp, var, division) VALUES (NULL, ?, ?, NULL, NULL, ?)";
								$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
								$rv = $sth->execute ($genus, $species, $division) || die "can't execute the query: $sth->errstr";
								$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ? AND subsp IS NULL AND var IS NULL";
								$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
								$rv = $sth->execute ($genus, $species) || die "can't execute the query: $sth->errstr";
								my $division_second_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
								$division_id = $division_second_table->[0][0]; 
								}
						}
					
						
					#look for locus
					$mystr = "SELECT locus_id FROM $loci WHERE name_long = ?";
					$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
					$rv = $sth->execute ($name_long) || die "can't execute the query: $sth->errstr";
					my $locus_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
					if(($#{$locus_table}) > -1){ #if exists...	
						$locus_id = $locus_table->[0][0]; 
						} else { #insert
							$mystr = "INSERT INTO $loci (locus_id, name_short, name_long, genome) VALUES (NULL, NULL, ?, NULL)";
							$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
							$rv = $sth->execute ($name_long) || die "can't execute the query: $sth->errstr";
							$mystr = "SELECT locus_id FROM $loci WHERE name_long = ?";
							$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
							$rv = $sth->execute ($name_long) || die "can't execute the query: $sth->errstr";
							my $second_locus_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
							$locus_id = $second_locus_table->[0][0]; 
							}

						
					#insert seq
					$mystr = "INSERT INTO $sequences (seq_id, taxon_id, locus_id, genbank_id, genbank_version, sequence, score) 
					VALUES (NULL, ?, ?, ?, ?, ?, ?)";   
					$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
					$rv = $sth->execute ($division_id, $locus_id, $genbank_id, $genbank_version, $seq, $score) || die "can't execute the query: $sth->errstr";
					$insert++;
					}	

			} else {
				#$genbank = "no";
				$label =~ tr/\>\#//d;
				@junk = split(/\_/, $label);
				$genbank_id = $junk[0];
				$genbank_version = "";
				$genus = $junk[1];
				$species = $junk[2];
				$subsp = "";
				$var = "";
				
				#look for taxon
				$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ?";
				$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
				$rv = $sth->execute ($genus, $species,) || die "can't execute the query: $sth->errstr";
				my $division_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
				if(($#{$division_table}) > -1){ #if exists...	
					$division_id = $division_table->[0][0]; 
					} else {#insert taxon
						$mystr = "INSERT INTO $taxa (taxon_id, genus, species, subsp, var, division) VALUES (NULL, ?, ?, NULL, NULL, NULL)";
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($genus, $species) || die "can't execute the query: $sth->errstr";
						$mystr = "SELECT taxon_id FROM $taxa WHERE genus = ? AND species = ?";
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($genus, $species) || die "can't execute the query: $sth->errstr";
						my $division_second_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
						$division_id = $division_second_table->[0][0]; 
						}
				
				#look for locus
				$mystr = "SELECT locus_id FROM $loci WHERE name_short = ?";
				$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
				$rv = $sth->execute ($ARGV[2]) || die "can't execute the query: $sth->errstr";
				my $locus_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
				if(($#{$locus_table}) > -1){ #if exists...	
					$locus_id = $locus_table->[0][0]; 
					} else { #insert
						$mystr = "INSERT INTO $loci (locus_id, name_short, name_long, genome) VALUES (NULL, ?, NULL, NULL)";
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($ARGV[2]) || die "can't execute the query: $sth->errstr";
						$mystr = "SELECT locus_id FROM $loci WHERE name_short = ?";
						$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
						$rv = $sth->execute ($ARGV[2]) || die "can't execute the query: $sth->errstr";
						my $second_locus_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
						$locus_id = $second_locus_table->[0][0]; 
						}
						
				#insert seq
				$mystr = "INSERT INTO $sequences (seq_id, taxon_id, locus_id, genbank_id, genbank_version, sequence, score) 
				VALUES (NULL, ?, ?, ?, ?, ?, ?)";   
				$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
				$rv = $sth->execute ($division_id, $locus_id, $genbank_id, $genbank_version, $seq, $score) || die "can't execute the query: $sth->errstr";
				$insert++;
				}		
		}	
}
exit;
