#!/usr/bin/perl

############################### This program is free software; you can redistribute it and/or modify
############################### it under the terms of the GNU General Public License as published by
############################### the Free Software Foundation; either version 2 of the License, or
############################### (at your option) any later version.
############################### 
############################### This program is distributed in the hope that it will be useful,
############################### but WITHOUT ANY WARRANTY; without even the implied warranty of
############################### MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
############################### GNU General Public License for more details.
############################### 
############################### You should have received a copy of the GNU General Public License
############################### along with this program; if not, write to the Free Software
############################### Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
###############################
###############################
###############################
############################### Copyright 2006 Damon P. Little

#start
use DBI;
my $db = "barcode";
my $host = "localhost";
my $id = "barcode";
my $pwd = "barcode";
$loci = "loci";
$sequences = "sequences";
$taxa = "taxa";
$oligos = "motifs";
$frequency = "freq10";

#get options
if($ARGV[0] eq "-e"){
	$option = "=";
	} elsif ($ARGV[0] eq "-l"){
		$option = "LIKE";
		} 
if($ARGV[1]){
	$locus = $ARGV[1]; 
	} else {
		$locus = "NULL";
		}
if($ARGV[2]){
	$number = $ARGV[2]
	} else {
		$number = "-a";
		}
	
#error trap
if($locus eq "NULL"){
	print "A Perl script for making a 0/1 matrix of motifs\n";
	print "USAGE: mysql2ss.pl -e[-l] locus -a[-h]\n";
	print "where -e specifies exact locus match; -l LIKE\n";
	print "-a all sequences; -h high scoring sequences\n";
	} else {
		
		#connect
		my $dbh = DBI->connect("DBI:mysql:$db:$host",$id,$pwd);
		if(!defined $dbh){
			die "Can't connect: $DBI::errstr \n";
			}
		
		#olig query
		$mystr = "SELECT motif FROM $oligos, $loci, $frequency 
				  WHERE $oligos.motif_id = $frequency.motif_id AND $frequency.locus_id = $loci.locus_id  
				  AND $loci.name_short $option ?	
				  AND $frequency.hits IS NOT NULL 
				  ORDER BY $oligos.motif_id";	
		$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute ($locus) || die "can't execute the query: $sth->errstr";
		my $oligo_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		
		$mystr = "SELECT count(motif) FROM $oligos, $loci, $frequency 
		  WHERE $oligos.motif_id = $frequency.motif_id AND $frequency.locus_id = $loci.locus_id  
		  AND $loci.name_short $option ?	
		  AND $frequency.hits IS NOT NULL 
		  ORDER BY $oligos.motif_id";	
		$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute ($locus) || die "can't execute the query: $sth->errstr";
		my $char_table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		$nchar = $char_table->[0][0];
		
		#seq query
		$mystr = "SELECT $sequences.genbank_id, $taxa.genus, $taxa.species, $taxa.subsp, $taxa.var, $sequences.sequence 
				  FROM $sequences, $taxa, $loci 
				  WHERE $taxa.taxon_id=$sequences.taxon_id AND $sequences.locus_id=$loci.locus_id 			  
				  AND $loci.name_short $option ?			  
				  ORDER BY $taxa.genus, $taxa.species, $sequences.score DESC";	
		$sth = $dbh->prepare($mystr) || die "Can't prepare $statement: $dbh->errstr\n";
		$rv = $sth->execute ($locus) || die "can't execute the query: $sth->errstr";
		my $table = $sth->fetchall_arrayref || die "$sth->errstr\n";
		
		$last_genus = "";
		$last_species = "";
		$last_seq = "";
		$seq_count = 0;
		$buffer = "";
							
		for $k (0 .. $#{$table}){					
			$seq_id = $table->[$k][0];
			$genus = $table->[$k][1];
			$species = $table->[$k][2];
			$subsp = $table->[$k][3];
			$var = $table->[$k][4];
			$seq = uc($table->[$k][5]);
		
			if((($genus eq $last_genus) && ($species eq $last_species) && ($number eq "-a") && ($seq ne $last_seq)) || ($genus ne $last_genus) || ($species ne $last_species)){
				$revcom = reverse($seq);
				$revcom =~ tr/ACGTNVDBHWMRKSY/TGCANBHVDWKYMSR/;
				$seq .= " " . $revcom;
				$buffer .= $seq_id . "          ";
				for $j (0 .. $#{$oligo_table}){
					$oligo = $oligo_table->[$j][0];
					if($seq =~ /$oligo/){
						$buffer .= "1";
						} else {
							$buffer .= "0";
							}
					}
				$buffer .= "\n";
				$seq_count++;
				}
			$last_genus = $genus;
			$last_species = $species;
			$last_seq = $seq;
			}
		$rc = $sth->finish;
		$dbh->disconnect;

		print "xread\n";
		print "'matrix template from mysql2ss.pl, add query sequence...'\n";
		print "$nchar $seq_count\n";
		print "$buffer\n";
		print ";\n";
		print "proc/;\n";
		$buffer = "#\n\$\n;\ncn ";
		$k = 0;
		for $j (0 .. $#{$oligo_table}){
			$oligo = $oligo_table->[$j][0];
			$buffer .= "{$k $oligo absent present;\n";
			$k++;
			}
		$buffer .= ";";
		print "$buffer\n";
		}	
exit;
